@@ -183,6 +183,13 @@ class Msf::Modules::Loader::Base
         causal_message:        'invalid module filename (must be lowercase alphanumeric snake case)'
       ))
       return false
+    rescue => e
+      load_error(module_path, Msf::Modules::Error.new(
+        module_path:           module_path,
+        module_reference_name: module_reference_name,
+        causal_message:        "unknown error #{e.message}"
+      ))
+      return false
     end
 
 
