@@ -19,6 +19,7 @@ module Msf
       postgres
       smb
       ssh
+      tcp
     ].freeze
     private_constant :SUPPORTED_SCHEMAS
 
@@ -58,7 +59,7 @@ module Msf
       return unless block_given?
 
       parse(@value, @datastore).each do |result|
-        block.call(result) if result.is_a?(Msf::DataStore)
+        block.call(result) if result.is_a?(Msf::DataStore) || result.is_a?(Msf::DataStoreWithFallbacks)
       end
 
       nil
@@ -92,7 +93,7 @@ module Msf
     # @return [Boolean] True if all items are valid, and there are at least some items present to iterate over. False otherwise.
     def valid?
       parsed_values = parse(@value, @datastore)
-      parsed_values.all? { |result| result.is_a?(Msf::DataStore) } && parsed_values.count > 0
+      parsed_values.all? { |result| result.is_a?(Msf::DataStore) || result.is_a?(Msf::DataStoreWithFallbacks) } && parsed_values.count > 0
     rescue StandardError => e
       elog('rhosts walker invalid', error: e)
       false
@@ -146,7 +147,7 @@ module Msf
               overrides = {}
               overrides['UNPARSED_RHOSTS'] = value
               overrides['RHOSTS'] = rhost[:address]
-              overrides['VHOST'] = rhost[:hostname] if datastore.options.include?('VHOST') && datastore['VHOST'].blank?
+              set_hostname(datastore, overrides, rhost[:hostname])
               results << datastore.merge(overrides)
             end
           end
@@ -169,6 +170,7 @@ module Msf
       result['RHOSTS'] = uri.hostname
       result['RPORT'] = (uri.port || 445) if datastore.options.include?('RPORT')
 
+      set_hostname(datastore, result, uri.hostname)
       # Handle users in the format:
       #   user
       #   domain;user
@@ -206,8 +208,6 @@ module Msf
     def parse_http_uri(value, datastore)
       uri = ::Addressable::URI.parse(value)
       result = {}
-      # nil VHOST for now, this value will be calculated and overridden later
-      result['VHOST'] = nil
 
       result['RHOSTS'] = uri.hostname
       is_ssl = %w[ssl https].include?(uri.scheme)
@@ -223,7 +223,9 @@ module Msf
         result['URI'] = target_uri if datastore.options.include?('URI')
       end
 
-      result['VHOST'] = uri.hostname unless Rex::Socket.is_ip_addr?(uri.hostname)
+      result['HttpQueryString'] = uri.query if datastore.options.include?('HttpQueryString')
+
+      set_hostname(datastore, result, uri.hostname)
       set_username(datastore, result, uri.user) if uri.user
       set_password(datastore, result, uri.password) if uri.password
 
@@ -249,6 +251,7 @@ module Msf
         result['DATABASE'] = uri.path[1..-1]
       end
 
+      set_hostname(datastore, result, uri.hostname)
       set_username(datastore, result, uri.user) if uri.user
       set_password(datastore, result, uri.password) if uri.password
       result
@@ -271,6 +274,8 @@ module Msf
       if datastore.options.include?('DATABASE') && has_database_specified
         result['DATABASE'] = uri.path[1..-1]
       end
+
+      set_hostname(datastore, result, uri.hostname)
       set_username(datastore, result, uri.user) if uri.user
       set_password(datastore, result, uri.password) if uri.password
 
@@ -290,6 +295,29 @@ module Msf
       result['RHOSTS'] = uri.hostname
       result['RPORT'] = uri.port || 22
 
+      set_hostname(datastore, result, uri.hostname)
+      set_username(datastore, result, uri.user) if uri.user
+      set_password(datastore, result, uri.password) if uri.password
+
+      result
+    end
+
+    # Parses a uri string such as tcp://user:password@example.com into a hash
+    # which can safely be merged with a [Msf::DataStore] datastore for setting options.
+    #
+    # @param value [String] the uri string
+    # @return [Hash] A hash where keys match the required datastore options associated with
+    #   the uri value
+    def parse_tcp_uri(value, datastore)
+      uri = ::Addressable::URI.parse(value)
+      result = {}
+
+      result['RHOSTS'] = uri.hostname
+      if uri.port
+        result['RPORT'] = uri.port
+      end
+
+      set_hostname(datastore, result, uri.hostname)
       set_username(datastore, result, uri.user) if uri.user
       set_password(datastore, result, uri.password) if uri.password
 
@@ -298,6 +326,12 @@ module Msf
 
     protected
 
+    def set_hostname(datastore, result, hostname)
+      hostname = Rex::Socket.is_ip_addr?(hostname) ? nil : hostname
+      result['RHOSTNAME'] = hostname if result['RHOSTNAME'].blank?
+      result['VHOST'] = hostname if datastore.options.include?('VHOST') && datastore['VHOST'].blank?
+    end
+
     def set_username(datastore, result, username)
       # Preference setting application specific values first
       username_set = false
