@@ -31,6 +31,22 @@ class Def_windows_advapi32
         ['LPVOID', 'hService', 'in'],
         ['PBLOB', 'lpServiceStatus', 'out']])
 
+    dll.add_function('BuildExplicitAccessWithNameA', 'VOID',[
+      ["PBLOB","pExplicitAccess","inout"],
+      ["PCHAR","pTrusteeName","in"],
+      ["DWORD","AccessPermissions","in"],
+      ["DWORD","AccessMode","in"],
+      ["DWORD","Inheritance","in"]
+    ])
+
+    dll.add_function('BuildExplicitAccessWithNameW', 'VOID',[
+      ["PBLOB","pExplicitAccess","inout"],
+      ["PWCHAR","pTrusteeName","in"],
+      ["DWORD","AccessPermissions","in"],
+      ["DWORD","AccessMode","in"],
+      ["DWORD","Inheritance","in"]
+    ])
+
     dll.add_function('CredEnumerateA', 'BOOL', [
         ['PCHAR', 'Filter', 'in'],
         ['DWORD', 'Flags', 'in'],
@@ -291,6 +307,27 @@ class Def_windows_advapi32
         ['LPVOID', 'hProv', 'in'],
         ['DWORD', 'dwFlags', 'in']])
 
+    dll.add_function('GetNamedSecurityInfoA', 'DWORD',[
+      ["PCHAR","pObjectName","in"],
+      ["DWORD","ObjectType","in"],
+      ["DWORD","SecurityInfo","in"],
+      ["PBLOB","ppsidOwner","out"],
+      ["PBLOB","ppsidGroup","out"],
+      ["PBLOB","ppDacl","out"],
+      ["PBLOB","ppSacl","out"],
+      ["PBLOB","ppSecurityDescriptor","out"]
+    ])
+
+    dll.add_function('GetNamedSecurityInfoW', 'DWORD',[
+      ["PWCHAR","pObjectName","in"],
+      ["DWORD","ObjectType","in"],
+      ["DWORD","SecurityInfo","in"],
+      ["PBLOB","ppsidOwner","out"],
+      ["PBLOB","ppsidGroup","out"],
+      ["PBLOB","ppDacl","out"],
+      ["PBLOB","ppSacl","out"],
+      ["PBLOB","ppSecurityDescriptor","out"]
+    ])
 
     # Function to open the Service Control Database
     dll.add_function('OpenSCManagerA','DWORD',[
@@ -2003,6 +2040,20 @@ class Def_windows_advapi32
       ["DWORD","dwAclInformationClass","in"],
       ])
 
+    dll.add_function('SetEntriesInAclA', 'DWORD',[
+      ["DWORD","cCountOfExplicitEntries","in"],
+      ["PBLOB","pListOfExplicitEntries","in"],
+      ["PBLOB","OldAcl","in"],
+      ["PBLOB","NewAcl","out"]
+    ])
+
+    dll.add_function('SetEntriesInAclW', 'DWORD',[
+      ["DWORD","cCountOfExplicitEntries","in"],
+      ["PBLOB","pListOfExplicitEntries","in"],
+      ["PBLOB","OldAcl","in"],
+      ["PBLOB","NewAcl","out"]
+    ])
+
     dll.add_function('SetFileSecurityA', 'BOOL',[
       ["PCHAR","lpFileName","in"],
       ["PBLOB","SecurityInformation","in"],
@@ -2021,6 +2072,26 @@ class Def_windows_advapi32
       ["PBLOB","SecurityDescriptor","in"],
       ])
 
+    dll.add_function('SetNamedSecurityInfoA', 'DWORD',[
+      ["PCHAR","pObjectName","in"],
+      ["DWORD","ObjectType","in"],
+      ["DWORD","SecurityInfo","in"],
+      ["PBLOB","psidOwner","in"],
+      ["PBLOB","psidGroup","in"],
+      ["PBLOB","pDacl","in"],
+      ["PBLOB","pSacl","in"]
+    ])
+
+    dll.add_function('SetNamedSecurityInfoW', 'DWORD',[
+      ["PWCHAR","pObjectName","in"],
+      ["DWORD","ObjectType","in"],
+      ["DWORD","SecurityInfo","in"],
+      ["PBLOB","psidOwner","in"],
+      ["PBLOB","psidGroup","in"],
+      ["PBLOB","pDacl","in"],
+      ["PBLOB","pSacl","in"]
+    ])
+
     dll.add_function('SetPrivateObjectSecurity', 'BOOL',[
       ["PBLOB","SecurityInformation","in"],
       ["PBLOB","ModificationDescriptor","in"],
